
from generallibrary import typeChecker, confineTo


class _GeneralVector:
    """ GeneralVector class that Vec2 and Vec inherits for shared functions.
        Todo: Move most methods to _GeneralVector. """
    def __init__(self, *axis, length):
        axis = tuple([float(n) if typeChecker(n, str, error=False) else n for n in axis if n is not None])

        axisLen = len(axis)
        if axisLen == 0:
            self.axis = tuple([0] * length)

        elif axisLen == 1:

            if typeChecker(axis[0], _GeneralVector, error=False):
                vector = axis[0]
                if len(vector.axis) != length:
                    raise AttributeError(f"{vector} was supplied as first argument but it's axis length is not {length}")

                self.axis = vector.axis + tuple()

            else:
                typeChecker(axis[0], float)
                self.axis = tuple([axis[0]] * length)

        elif axisLen == length:
            self.axis = axis

        else:
            raise AttributeError(f"Could not handle supplied axis {axis} for {self.__class__.__name__}")

        for n in self.axis:
            typeChecker(n, float)

    def __getitem__(self, item):
        return self.axis[item]

    def sanitize(self, ints=False, positive=False, positiveOrZero=False, negative=False, negativeOrZero=False, minimum=None, maximum=None):
        """
        Sanitize this vector with a bunch of optional flags.

        :param generalvector.Vec or generalvector.Vec2 self:
        :param ints:
        :param positive:
        :param positiveOrZero:
        :param negative:
        :param negativeOrZero:
        :param generalvector.Vec or generalvector.Vec2 minimum:
        :param generalvector.Vec or generalvector.Vec2 maximum:
        :return: self
        """
        if ints and self != self.round():
            raise ValueError(f"{self} failed 'ints' sanitizing")

        if positive and not self > 0:
            raise ValueError(f"{self} failed 'positive' sanitizing")
        if positiveOrZero and not self >= 0:
            raise ValueError(f"{self} failed 'positiveOrZero' sanitizing")

        if negative and not self < 0:
            raise ValueError(f"{self} failed 'negative' sanitizing")
        if negativeOrZero and not self <= 0:
            raise ValueError(f"{self} failed 'negativeOrZero' sanitizing")

        if minimum and not self >= minimum:
            raise ValueError(f"{self} failed 'minimum' sanitizing")
        if maximum and not self <= maximum:
            raise ValueError(f"{self} failed 'maximum' sanitizing")

        if ints:
            return self.round()
        else:
            return self

    def confineTo(self, pos, size, margin=0):
        """
        Confine this vector to an area, but unlike clamp it subtracts axis * n to create an 'infinite' area effect.

        :param generalvector.Vec or generalvector.Vec2 self:
        :param generalvector.Vec or generalvector.Vec2 pos: Lowest point of area
        :param generalvector.Vec or generalvector.Vec2 size: Size of area, has to be positive or zero
        :param float margin: Margin of confinement
        """
        pos = self.__class__(pos)
        size = self.__class__(size).sanitize(positiveOrZero=True)
        maximum = pos + size

        return self.__class__(*[confineTo(axis, pos[i], maximum[i], margin) for i, axis in enumerate(self.axis)])

    def absolute(self):
        """
        Return this vector with absolute values.

        :param generalvector.Vec or generalvector.Vec2 self:
        """
        return self.__class__(*[abs(axis) for axis in self.axis])
















import unittest
from generalvector.vector import Vec

class VecTest(unittest.TestCase):
    def test_vec(self):
        self.assertRaises(ValueError, Vec, "")
        self.assertRaises(ValueError, Vec, 5, 2, "")
        self.assertRaises(ValueError, Vec, 5, "", 2)

        self.assertRaises(AttributeError, Vec, 2, 5)


        self.assertEqual(Vec(5.2, 2, 3), Vec("5.2", "2", 3))

        self.assertEqual(Vec(None).x, Vec(0))
        self.assertEqual(Vec(None, 5).x, 5)
        self.assertEqual(Vec(5, None).x, 5)
        self.assertEqual(Vec(5, 2, 1).x, 5)
        self.assertEqual(Vec(5, 2, 1).y, 2)
        self.assertEqual(Vec(5, 2, 1).z, 1)

        self.assertEqual(Vec(5.2, 2.5, 1.4).x, 5.2)
        self.assertEqual(Vec(5.2, 2.5, 1.4).y, 2.5)
        self.assertEqual(Vec(5.2, 2.5, 1.4).z, 1.4)

        self.assertEqual(Vec(5).x, 5)
        self.assertEqual(Vec(5).y, 5)
        self.assertEqual(Vec(5).z, 5)

        self.assertEqual(Vec(5, 2, 1), Vec(Vec(5, 2, 1)))

    def test_equal(self):
        self.assertRaises(TypeError, Vec.__eq__, "")

        self.assertTrue(Vec(1) == Vec(1))
        self.assertTrue(Vec(1, 2, 3) == Vec(1, 2, 3))
        self.assertTrue(Vec(1, 1, 1) == 1)
        self.assertTrue(Vec(5.2, 5.2, 5.2) == 5.2)

        self.assertFalse(Vec(1, 2, 3) == Vec(1, 3, 3))
        self.assertFalse(Vec(1, 2, 3) == Vec(2, 2, 3))

        self.assertFalse(Vec(1, 1, 3) == 2)
        self.assertFalse(Vec(5.2, 5.2, 3) == 5)

        self.assertFalse(Vec(5.2, 5.2, 5.2) is None)
        self.assertFalse(Vec(5.2, 5.2, 5.2) == "test")

    def test_add(self):
        self.assertRaises(TypeError, Vec.__add__, "")
        self.assertRaises(TypeError, Vec.__add__, False)

        self.assertEqual(Vec(5, 2, 3) + Vec(2, 4, 3), Vec(7, 6, 6))
        self.assertEqual(Vec(5, 2, 3) + Vec(2.5, 4.5, 3.5), Vec(7.5, 6.5, 6.5))
        self.assertEqual(Vec(5, 2, 3) + Vec(-2, -4, -3), Vec(3, -2, 0))

        self.assertEqual(Vec(5, 2, 3) + 3, Vec(8, 5, 6))
        self.assertEqual(Vec(5, 2, 3) + -3, Vec(2, -1, 0))
        self.assertEqual(Vec(5, 2, 3) + 3.5, Vec(8.5, 5.5, 6.5))

    def test_sub(self):
        self.assertRaises(TypeError, Vec.__sub__, "")
        self.assertRaises(TypeError, Vec.__sub__, True)

        self.assertEqual(Vec(5, 2, 3) - Vec(2, 4, 3), Vec(3, -2, 0))
        self.assertEqual(Vec(5, 2, 3) - Vec(-2, -4, -3), Vec(7, 6, 6))
        self.assertEqual(Vec(5, 2, 3) - 3, Vec(2, -1, 0))
        self.assertEqual(Vec(5, 2, 3) - -3, Vec(8, 5, 6))
        self.assertEqual(Vec(5, 2, 3) - 3.5, Vec(1.5, -1.5, -0.5))

    def test_mul(self):
        self.assertRaises(TypeError, Vec.__mul__, "")
        self.assertRaises(TypeError, Vec.__mul__, True)

        self.assertEqual(Vec(5, 2, 3) * 2, Vec(10, 4, 6))
        self.assertEqual(Vec(5, 2, 3) * 2.5, Vec(12.5, 5, 7.5))

        self.assertEqual(Vec(5, 2, 3) * Vec(2, 1, 3), Vec(10, 2, 9))

    def test_div(self):
        self.assertRaises(TypeError, Vec.__truediv__, "")
        self.assertRaises(TypeError, Vec.__truediv__, False)

        self.assertEqual(Vec(5, 2, 3) / 2, Vec(2.5, 1, 1.5))
        self.assertEqual(Vec(10, 5, 7.5) / 2.5, Vec(4, 2, 3))

        self.assertEqual(Vec(10, 5, 7.5) / Vec(5, 2, 2.5), Vec(2, 2.5, 3))

    def test_length(self):
        self.assertEqual(Vec(10, 0, 0).length(), 10)
        self.assertEqual(Vec(0, 10, 0).length(), 10)
        self.assertEqual(Vec(0, 0, 10).length(), 10)
        self.assertEqual(Vec(0, 0.2, 0).length(), 0.2)
        self.assertEqual(Vec(0, 0, 0).length(), 0)

    def test_normalized(self):
        self.assertEqual(Vec(10, 0, 0).normalized(), Vec(1, 0, 0))
        self.assertEqual(Vec(0, 10, 0).normalized(), Vec(0, 1, 0))
        self.assertEqual(Vec(0, 0.2, 0).normalized(), Vec(0, 1, 0))
        self.assertEqual(Vec(0, 0, 0).normalized(), Vec(0, 0, 0))
        self.assertEqual(Vec(0, -10, 0).normalized(), Vec(0, -1, 0))

    def test_round(self):
        self.assertEqual(Vec(5, 2, 1), Vec(5.3, 1.5, 1.4).round())

    def test_random(self):
        randvec = Vec.random(5)
        self.assertGreaterEqual(randvec.x, 0)
        self.assertGreaterEqual(randvec.y, 0)
        self.assertGreaterEqual(randvec.z, 0)
        self.assertLessEqual(randvec.x, 5)
        self.assertLessEqual(randvec.y, 5)
        self.assertLessEqual(randvec.z, 5)

        randvec = Vec.random(3, 5)
        self.assertGreaterEqual(randvec.x, 3)
        self.assertGreaterEqual(randvec.y, 3)
        self.assertGreaterEqual(randvec.z, 3)
        self.assertLessEqual(randvec.x, 5)
        self.assertLessEqual(randvec.y, 5)
        self.assertLessEqual(randvec.z, 5)

        randvec = Vec.random(Vec(-1, 2, 6))
        self.assertGreaterEqual(randvec.x, -1)
        self.assertGreaterEqual(randvec.y, 0)
        self.assertGreaterEqual(randvec.z, 0)
        self.assertLessEqual(randvec.x, 0)
        self.assertLessEqual(randvec.y, 2)
        self.assertLessEqual(randvec.z, 6)

        randvec = Vec.random(Vec(4, 2, 6), 8)
        self.assertGreaterEqual(randvec.x, 4)
        self.assertGreaterEqual(randvec.y, 2)
        self.assertGreaterEqual(randvec.z, 6)
        self.assertLessEqual(randvec.x, 8)
        self.assertLessEqual(randvec.y, 8)
        self.assertLessEqual(randvec.z, 8)

        randvec = Vec.random(8, Vec(4, 2, 6))
        self.assertGreaterEqual(randvec.x, 4)
        self.assertGreaterEqual(randvec.y, 2)
        self.assertGreaterEqual(randvec.z, 6)
        self.assertLessEqual(randvec.x, 8)
        self.assertLessEqual(randvec.y, 8)
        self.assertLessEqual(randvec.z, 8)

        randvec = Vec.random(Vec(3.5, 3, 10), Vec(4, 2, 6))
        self.assertGreaterEqual(randvec.x, 3.5)
        self.assertGreaterEqual(randvec.y, 2)
        self.assertGreaterEqual(randvec.z, 6)
        self.assertLessEqual(randvec.x, 4)
        self.assertLessEqual(randvec.y, 3)
        self.assertLessEqual(randvec.z, 10)

    def test_min(self):
        self.assertEqual(Vec(5, 2, 1), Vec(8, 2, 10).min(Vec(5, 5, 1)))
        self.assertEqual(Vec(5, 2, 1), Vec(10).min(Vec(5, 2, 1)))
        self.assertEqual(Vec(5, 2, -1), Vec(5, 2, -1).min(Vec(99)))
        self.assertEqual(Vec(5, 5, 5), Vec(6.5).min(Vec(5)))
        self.assertEqual(Vec(5.5, 5.5, 5.5), Vec(6.5).min(Vec(5.5)))

    def test_max(self):
        self.assertEqual(Vec(8, 5, 10), Vec(8, 2, 10).max(Vec(5, 5, 1)))
        self.assertEqual(Vec(10), Vec(10).max(Vec(5, 2, 1)))
        self.assertEqual(Vec(99), Vec(5, 2, -1).max(Vec(99)))
        self.assertEqual(Vec(6.5), Vec(6.5).max(Vec(5)))
        self.assertEqual(Vec(6.5), Vec(6.5).max(Vec(5.5)))

    def test_clamp(self):
        self.assertEqual(Vec(5, 2, 1), Vec(6, 2, 1).clamp(0, 5))
        self.assertEqual(Vec(5, 2, 1), Vec(0).clamp(Vec(5, 2, 1), 5))
        self.assertEqual(Vec(-3, 2, 1), Vec(0).clamp(Vec(-3, 2, 1), Vec(-3, 2, 1)))
        self.assertEqual(Vec(-3, 2, 1), Vec(0).clamp(Vec(-30, 2, 1), Vec(-3, 20, 10)))

    def test_inrange(self):
        self.assertEqual(True, Vec(5).inrange(Vec(4), Vec(6)))
        self.assertEqual(True, Vec(5).inrange(Vec(5), Vec(5)))
        self.assertEqual(True, Vec(2, 3, 4).inrange(Vec(1, 2, 3), Vec(5)))

        self.assertEqual(False, Vec(2, 3, 4).inrange(Vec(1, 2, 3), Vec(5, 5, 3)))
        self.assertEqual(False, Vec(2, 3, 4).inrange(Vec(1, 2, 3), Vec(5, 2, 5)))
        self.assertEqual(False, Vec(2, 3, 4).inrange(Vec(1, 2, 3), Vec(1, 5, 5)))

        self.assertEqual(False, Vec(2, 3, 4).inrange(Vec(3, 2, 3), Vec(5)))
        self.assertEqual(False, Vec(2, 3, 4).inrange(Vec(1, 4, 3), Vec(5)))
        self.assertEqual(False, Vec(2, 3, 4).inrange(Vec(1, 2, 5), Vec(5)))

    def test_hex(self):
        self.assertEqual("#c8c800", Vec(200, 200, 0).hex())
        self.assertEqual("#ff0000", Vec(255, 0, 0).hex())
        self.assertEqual("#ff0000", Vec(300, -2, -100).hex())
        self.assertEqual("#ff1632", Vec(255, 22, 50).hex())

    def test_lessThan(self):
        self.assertEqual(True, Vec(10) < Vec(11))
        self.assertEqual(False, Vec(10) < Vec(10))
        self.assertEqual(True, Vec(10, 10, 10) < Vec(11, 15, 14))
        self.assertEqual(False, Vec(10, 10, 10) < Vec(9, 15, 14))
        self.assertEqual(False, Vec(10, 10, 10) < Vec(10, 15, 14))
        self.assertEqual(False, Vec(10, 10, 10) < Vec(9, 9, 9))

    def test_greaterThan(self):
        self.assertEqual(False, Vec(10) > Vec(11))
        self.assertEqual(False, Vec(10) > Vec(10))
        self.assertEqual(False, Vec(10, 10, 10) > Vec(11, 15, 14))
        self.assertEqual(False, Vec(10, 10, 10) > Vec(9, 15, 14))
        self.assertEqual(False, Vec(10, 10, 10) > Vec(10, 5, 4))
        self.assertEqual(True, Vec(10, 10, 10) > Vec(9, 9, 9))

    def test_lessThanOrEqual(self):
        self.assertEqual(True, Vec(5) <= Vec(5))
        self.assertEqual(True, Vec(4) <= Vec(5))
        self.assertEqual(False, Vec(6) <= Vec(5))
        self.assertEqual(True, Vec(10, 10, 10) <= Vec(10, 15, 14))

    def test_greaterThanOrEqual(self):
        self.assertEqual(True, Vec(5) >= Vec(5))
        self.assertEqual(True, Vec(6) >= Vec(5))
        self.assertEqual(False, Vec(4) >= Vec(5))
        self.assertEqual(True, Vec(10, 10, 10) >= Vec(10, 9, 7))

    def test_range(self):
        self.assertRaises(ValueError, Vec(0, -1, 0).range, Vec(-1, -1, 1))
        self.assertRaises(ValueError, Vec(0, -1, 0).range, Vec(-1, 1, 1))
        self.assertRaises(ValueError, Vec(0, -1.2, 0).range, Vec(1, 1, 1))
        self.assertRaises(ValueError, Vec(0, 0, 0).range, Vec(1, 1.1, 1))

        self.assertEqual([Vec(0, 0, 0)], Vec(0, 0, 0).range(Vec(1, 1, 1)))
        self.assertEqual([Vec(0, 0, 0), Vec(1, 0, 0)], Vec(0, 0, 0).range(Vec(2, 1, 1)))
        self.assertEqual([Vec(0, 0, 0), Vec(0, 1, 0), Vec(0, 0, 1), Vec(0, 1, 1)], Vec(0, 0, 0).range(Vec(1, 2, 2)))
        self.assertEqual([Vec(0, 1, 0), Vec(0, 2, 0), Vec(0, 3, 0)], Vec(0, 1, 0).range(Vec(1, 3, 1)))
        self.assertEqual([Vec(2, 2, 2), Vec(3, 2, 2), Vec(2, 3, 2), Vec(3, 3, 2), Vec(2, 2, 3), Vec(3, 2, 3), Vec(2, 3, 3), Vec(3, 3, 3)], Vec(2).range(Vec(2)))

        self.assertEqual([], Vec(2).range(Vec(0, 1, 2)))
        self.assertEqual([], Vec(2).range(Vec(1, 0, 2)))
        self.assertEqual([], Vec(2).range(Vec(1, 3, 0)))
        self.assertEqual([], Vec(2).range(Vec(0, 0, 0)))

    def test_distance(self):
        self.assertEqual(5, Vec(10, 5, 0).distance(Vec(15, 5, 0)))
        self.assertEqual(5, Vec(10, 5, 0).distance(Vec(5, 5, 0)))
        self.assertEqual(0, Vec(10, 5, 0).distance(Vec(10, 5, 0)))
        self.assertEqual(2, Vec(-2, 0, 0).distance(Vec(0, 0, 0)))
        self.assertEqual(2, Vec(0, 0, 0).distance(Vec(-2, 0, 0)))
        self.assertEqual(2, Vec(0, 0, 2).distance(Vec(0, 0, 0)))

    def test_sanitize(self):
        self.assertRaises(ValueError, Vec(0.5).sanitize, ints=True)
        self.assertRaises(ValueError, Vec(0).sanitize, positive=True)
        self.assertRaises(ValueError, Vec(-1).sanitize, positive=True)
        self.assertRaises(ValueError, Vec(-1).sanitize, positiveOrZero=True)
        self.assertRaises(ValueError, Vec(0).sanitize, negative=True)
        self.assertRaises(ValueError, Vec(1).sanitize, negative=True)
        self.assertRaises(ValueError, Vec(1).sanitize, negativeOrZero=True)
        self.assertRaises(ValueError, Vec(1).sanitize, minimum=Vec(2))
        self.assertRaises(ValueError, Vec(1).sanitize, maximum=Vec(0))

        self.assertTrue(Vec(1).sanitize(ints=True))
        self.assertTrue(Vec(1).sanitize(positive=True))
        self.assertTrue(Vec(0).sanitize(positiveOrZero=True))
        self.assertTrue(Vec(1).sanitize(positiveOrZero=True))
        self.assertTrue(Vec(-1).sanitize(negative=True))
        self.assertTrue(Vec(0).sanitize(negativeOrZero=True))
        self.assertTrue(Vec(-1).sanitize(negativeOrZero=True))
        self.assertTrue(Vec(1).sanitize(minimum=Vec(1)))
        self.assertTrue(Vec(1).sanitize(maximum=Vec(1)))

    def test_confineTo(self):
        self.assertEqual(Vec(1, 1, 1), Vec(11, 11, 11).confineTo(Vec(0, 0, 0), Vec(10, 10, 10)))
        self.assertEqual(Vec(1, 1, 1), Vec(11, 11, 11).confineTo(Vec(-1, 0, 0), Vec(10, 10, 10)))
        self.assertEqual(Vec(2, 7, 5), Vec(22, -13, 5).confineTo(Vec(0, 0, 0), Vec(10, 10, 10)))

        self.assertEqual(Vec(0), Vec(2).confineTo(Vec(0), Vec(1), margin=0.5))

    def test_absolute(self):
        self.assertEqual(Vec(1, 2, 3), Vec(1, 2, 3).absolute())
        self.assertEqual(Vec(1, 2, 3), Vec(-1, 2, 3).absolute())
        self.assertEqual(Vec(1, 2, 3), Vec(1, -2, 3).absolute())
        self.assertEqual(Vec(1, 2, 3), Vec(1, 2, -3).absolute())
        self.assertEqual(Vec(1, 2, 3), Vec(-1, -2, -3).absolute())
        self.assertEqual(Vec(1.1, 2, 3), Vec(-1.1, -2, -3).absolute())




















import unittest
from generalvector.vector2 import Vec2

class Vec2Test(unittest.TestCase):
    def test_vec2(self):
        self.assertRaises(ValueError, Vec2, "")
        self.assertEqual(Vec2(5.2, 2), Vec2("5.2", "2"))

        self.assertEqual(Vec2(None).x, Vec2(0))
        self.assertEqual(Vec2(None, 5).x, 5)
        self.assertEqual(Vec2(5, None).x, 5)
        self.assertEqual(Vec2(5, 2).x, 5)
        self.assertEqual(Vec2(5, 2).y, 2)
        self.assertEqual(Vec2(5.2, 2).x, 5.2)
        self.assertEqual(Vec2(5.2, 2.5).y, 2.5)
        self.assertEqual(Vec2(5).x, 5)
        self.assertEqual(Vec2(5).y, 5)

        self.assertEqual(Vec2(5, 2), Vec2(Vec2(5, 2)))

    def test_equal(self):
        self.assertRaises(TypeError, Vec2.__eq__, "")

        self.assertTrue(Vec2(1) == Vec2(1))
        self.assertTrue(Vec2(1, 2) == Vec2(1, 2))
        self.assertTrue(Vec2(1, 1) == 1)
        self.assertTrue(Vec2(5.2, 5.2) == 5.2)

        self.assertFalse(Vec2(5.2, 5.2) != 5.2)

        self.assertFalse(Vec2(1, 2) == Vec2(1, 3))
        self.assertFalse(Vec2(1, 2) == Vec2(2, 2))
        self.assertFalse(Vec2(1, 1) == 2)
        self.assertFalse(Vec2(5.2, 5.2) == 5)

        self.assertTrue(Vec2(5.2, 5.2) != 5)

        self.assertFalse(Vec2(5.2, 5.2) is None)
        self.assertFalse(Vec2(5.2, 5.2) == "test")

    def test_add(self):
        self.assertRaises(TypeError, Vec2.__add__, "")
        self.assertRaises(TypeError, Vec2.__add__, False)

        self.assertEqual(Vec2(5, 2) + Vec2(2, 4), Vec2(7, 6))
        self.assertEqual(Vec2(5, 2) + Vec2(2.5, 4.5), Vec2(7.5, 6.5))
        self.assertEqual(Vec2(5, 2) + Vec2(-2, -4), Vec2(3, -2))
        self.assertEqual(Vec2(5, 2) + 3, Vec2(8, 5))
        self.assertEqual(Vec2(5, 2) + -3, Vec2(2, -1))
        self.assertEqual(Vec2(5, 2) + 3.5, Vec2(8.5, 5.5))

    def test_sub(self):
        self.assertRaises(TypeError, Vec2.__sub__, "")
        self.assertRaises(TypeError, Vec2.__sub__, True)

        self.assertEqual(Vec2(5, 2) - Vec2(2, 4), Vec2(3, -2))
        self.assertEqual(Vec2(5, 2) - Vec2(-2, -4), Vec2(7, 6))
        self.assertEqual(Vec2(5, 2) - 3, Vec2(2, -1))
        self.assertEqual(Vec2(5, 2) - -3, Vec2(8, 5))
        self.assertEqual(Vec2(5, 2) - 3.5, Vec2(1.5, -1.5))

    def test_mul(self):
        self.assertRaises(TypeError, Vec2.__mul__, "")
        self.assertRaises(TypeError, Vec2.__mul__, True)

        self.assertEqual(Vec2(5, 2) * 2, Vec2(10, 4))
        self.assertEqual(Vec2(5, 2) * 2.5, Vec2(12.5, 5))

        self.assertEqual(Vec2(5, 2) * Vec2(2, 1), Vec2(10, 2))

    def test_div(self):
        self.assertRaises(TypeError, Vec2.__truediv__, "")
        self.assertRaises(TypeError, Vec2.__truediv__, False)

        self.assertEqual(Vec2(5, 2) / 2, Vec2(2.5, 1))
        self.assertEqual(Vec2(10, 5) / 2.5, Vec2(4, 2))

        self.assertEqual(Vec2(10, 5) / Vec2(5, 2), Vec2(2, 2.5))

    def test_length(self):
        self.assertEqual(Vec2(10, 0).length(), 10)
        self.assertEqual(Vec2(0, 10).length(), 10)
        self.assertEqual(Vec2(0, 0.2).length(), 0.2)
        self.assertEqual(Vec2(0, 0).length(), 0)

    def test_normalized(self):
        self.assertEqual(Vec2(10, 0).normalized(), Vec2(1, 0))
        self.assertEqual(Vec2(0, 10).normalized(), Vec2(0, 1))
        self.assertEqual(Vec2(0, 0.2).normalized(), Vec2(0, 1))
        self.assertEqual(Vec2(0, 0).normalized(), Vec2(0, 0))
        self.assertEqual(Vec2(0, -10).normalized(), Vec2(0, -1))

    def test_round(self):
        self.assertEqual(Vec2(5, 2), Vec2(5.3, 1.5).round())

    def test_random(self):
        randvec = Vec2.random(5)
        self.assertGreaterEqual(randvec.x, 0)
        self.assertGreaterEqual(randvec.y, 0)
        self.assertLessEqual(randvec.x, 5)
        self.assertLessEqual(randvec.y, 5)

        randvec = Vec2.random(3, 5)
        self.assertGreaterEqual(randvec.x, 3)
        self.assertGreaterEqual(randvec.y, 3)
        self.assertLessEqual(randvec.x, 5)
        self.assertLessEqual(randvec.y, 5)

        randvec = Vec2.random(Vec2(-1, 2))
        self.assertGreaterEqual(randvec.x, -1)
        self.assertGreaterEqual(randvec.y, 0)
        self.assertLessEqual(randvec.x, 0)
        self.assertLessEqual(randvec.y, 2)

        randvec = Vec2.random(Vec2(4, 2), 8)
        self.assertGreaterEqual(randvec.x, 4)
        self.assertGreaterEqual(randvec.y, 2)
        self.assertLessEqual(randvec.x, 8)
        self.assertLessEqual(randvec.y, 8)

        randvec = Vec2.random(8, Vec2(4, 2))
        self.assertGreaterEqual(randvec.x, 4)
        self.assertGreaterEqual(randvec.y, 2)
        self.assertLessEqual(randvec.x, 8)
        self.assertLessEqual(randvec.y, 8)

        randvec = Vec2.random(Vec2(3.5, 3), Vec2(4, 2))
        self.assertGreaterEqual(randvec.x, 3.5)
        self.assertGreaterEqual(randvec.y, 2)
        self.assertLessEqual(randvec.x, 4)
        self.assertLessEqual(randvec.y, 3)

    def test_min(self):
        self.assertEqual(Vec2(5, 2), Vec2(8, 2).min(Vec2(5, 5)))
        self.assertEqual(Vec2(5, 2), Vec2(10).min(Vec2(5, 2)))
        self.assertEqual(Vec2(5, 2), Vec2(5, 2).min(Vec2(99)))
        self.assertEqual(Vec2(5, 5), Vec2(6.5).min(Vec2(5)))
        self.assertEqual(Vec2(5.5, 5.5), Vec2(6.5).min(Vec2(5.5)))

    def test_max(self):
        self.assertEqual(Vec2(8, 5), Vec2(8, 2).max(Vec2(5, 5)))
        self.assertEqual(Vec2(10), Vec2(10).max(Vec2(5, 2)))
        self.assertEqual(Vec2(99), Vec2(5, 2).max(Vec2(99)))
        self.assertEqual(Vec2(6.5), Vec2(6.5).max(Vec2(5)))
        self.assertEqual(Vec2(6.5), Vec2(6.5).max(Vec2(5.5)))

    def test_clamp(self):
        self.assertEqual(Vec2(5, 2), Vec2(6, 2).clamp(0, 5))
        self.assertEqual(Vec2(5, 2), Vec2(0).clamp(Vec2(5, 2), 5))
        self.assertEqual(Vec2(-3, 2), Vec2(0).clamp(Vec2(-3, 2), Vec2(-3, 2)))
        self.assertEqual(Vec2(-3, 2), Vec2(0).clamp(Vec2(-30, 2), Vec2(-3, 20)))

    def test_inrange(self):
        self.assertEqual(True, Vec2(5).inrange(Vec2(4), Vec2(6)))
        self.assertEqual(True, Vec2(5).inrange(Vec2(5), Vec2(5)))
        self.assertEqual(True, Vec2(2, 3).inrange(Vec2(1, 2), Vec2(5)))

        self.assertEqual(False, Vec2(2, 3).inrange(Vec2(1, 2), Vec2(5, 2)))
        self.assertEqual(False, Vec2(2, 3).inrange(Vec2(1, 2), Vec2(1, 5)))

        self.assertEqual(False, Vec2(2, 3).inrange(Vec2(3, 2), Vec2(5)))
        self.assertEqual(False, Vec2(2, 3).inrange(Vec2(1, 4), Vec2(5)))

    def test_lessThan(self):
        self.assertEqual(True, Vec2(10) < Vec2(11))
        self.assertEqual(True, Vec2(10, 10) < Vec2(11, 15))
        self.assertEqual(False, Vec2(10, 10) < Vec2(9, 15))
        self.assertEqual(False, Vec2(10, 10) < Vec2(9, 9))
        self.assertEqual(False, Vec2(10, 10) < Vec2(10, 9))

    def test_greaterThan(self):
        self.assertEqual(False, Vec2(10) > Vec2(11))
        self.assertEqual(False, Vec2(10, 10) > Vec2(11, 15))
        self.assertEqual(False, Vec2(10, 10) > Vec2(9, 15))
        self.assertEqual(False, Vec2(10, 10) > Vec2(10, 15))
        self.assertEqual(True, Vec2(10, 10) > Vec2(9, 9))

    def test_lessThanOrEqual(self):
        self.assertEqual(True, Vec2(5) <= Vec2(5))
        self.assertEqual(True, Vec2(4) <= Vec2(5))
        self.assertEqual(True, Vec2(4, 4) <= Vec2(4, 5))
        self.assertEqual(False, Vec2(6) <= Vec2(5))

    def test_greaterThanOrEqual(self):
        self.assertEqual(True, Vec2(5) >= Vec2(5))
        self.assertEqual(True, Vec2(6) >= Vec2(5))
        self.assertEqual(True, Vec2(6, 6) >= Vec2(5, 6))
        self.assertEqual(False, Vec2(4) >= Vec2(5))

    def test_range(self):
        self.assertRaises(ValueError, Vec2(0, -1).range, Vec2(-1, -1))
        self.assertRaises(ValueError, Vec2(0, -1).range, Vec2(-1, 1))
        self.assertRaises(ValueError, Vec2(0, -1.2).range, Vec2(1, 1))
        self.assertRaises(ValueError, Vec2(0, 0).range, Vec2(1, 1.1))

        self.assertEqual([Vec2(0, 0)], Vec2(0, 0).range(Vec2(1, 1)))
        self.assertEqual([Vec2(0, 0), Vec2(1, 0), Vec2(0, 1), Vec2(1, 1)], Vec2(0, 0).range(Vec2(2, 2)))
        self.assertEqual([Vec2(-1, 0), Vec2(0, 0), Vec2(-1, 1), Vec2(0, 1)], Vec2(-1, 0).range(Vec2(2, 2)))
        self.assertEqual([Vec2(1, 1), Vec2(2, 1), Vec2(3, 1)], Vec2(1, 1).range(Vec2(3, 1)))
        self.assertEqual([Vec2(0, -1), Vec2(0), Vec2(0, 1)], Vec2(0, -1).range(Vec2(1, 3)))
        self.assertEqual([], Vec2(0, -1).range(Vec2(0, 3)))
        self.assertEqual([], Vec2(0, -1).range(Vec2(2, 0)))
        self.assertEqual([], Vec2(0, -1).range(Vec2(0, 0)))

    def test_distance(self):
        self.assertEqual(5, Vec2(10, 5).distance(Vec2(15, 5)))
        self.assertEqual(5, Vec2(10, 5).distance(Vec2(5, 5)))
        self.assertEqual(0, Vec2(10, 5).distance(Vec2(10, 5)))
        self.assertEqual(2, Vec2(-2, 0).distance(Vec2(0, 0)))
        self.assertEqual(2, Vec2(0, 0).distance(Vec2(-2, 0)))

    def test_sanitize(self):
        self.assertRaises(ValueError, Vec2(0.5).sanitize, ints=True)
        self.assertRaises(ValueError, Vec2(0).sanitize, positive=True)
        self.assertRaises(ValueError, Vec2(-1).sanitize, positive=True)
        self.assertRaises(ValueError, Vec2(-1).sanitize, positiveOrZero=True)
        self.assertRaises(ValueError, Vec2(0).sanitize, negative=True)
        self.assertRaises(ValueError, Vec2(1).sanitize, negative=True)
        self.assertRaises(ValueError, Vec2(1).sanitize, negativeOrZero=True)
        self.assertRaises(ValueError, Vec2(1).sanitize, minimum=Vec2(2))
        self.assertRaises(ValueError, Vec2(1).sanitize, maximum=Vec2(0))

        self.assertTrue(Vec2(1).sanitize(ints=True))
        self.assertTrue(Vec2(1).sanitize(positive=True))
        self.assertTrue(Vec2(0).sanitize(positiveOrZero=True))
        self.assertTrue(Vec2(1).sanitize(positiveOrZero=True))
        self.assertTrue(Vec2(-1).sanitize(negative=True))
        self.assertTrue(Vec2(0).sanitize(negativeOrZero=True))
        self.assertTrue(Vec2(-1).sanitize(negativeOrZero=True))
        self.assertTrue(Vec2(1).sanitize(minimum=Vec2(1)))
        self.assertTrue(Vec2(1).sanitize(maximum=Vec2(1)))

    def test_confineTo(self):
        self.assertEqual(Vec2(1, 1), Vec2(11, 11).confineTo(Vec2(0, 0), Vec2(10, 10)))
        self.assertEqual(Vec2(1, 1), Vec2(11, 11).confineTo(Vec2(-1, 0), Vec2(10, 10)))
        self.assertEqual(Vec2(2, 7), Vec2(22, -13).confineTo(Vec2(0, 0), Vec2(10, 10)))

        self.assertEqual(Vec2(0), Vec2(2).confineTo(Vec2(0), Vec2(1), margin=0.5))

    def test_absolute(self):
        self.assertEqual(Vec2(1, 2), Vec2(1, 2).absolute())
        self.assertEqual(Vec2(1, 2), Vec2(-1, 2).absolute())
        self.assertEqual(Vec2(1, 2), Vec2(1, -2).absolute())
        self.assertEqual(Vec2(1, 2), Vec2(-1, -2).absolute())
        self.assertEqual(Vec2(1.1, 2), Vec2(-1.1, -2).absolute())










































from generallibrary import clamp, inrange, typeChecker
from generalvector.general import _GeneralVector

from math import sqrt
import random


class Vec2(_GeneralVector):
    """
    Immutable vector2.
    """
    def __init__(self, x=None, y=None):
        _GeneralVector.__init__(self, x, y, length=2)

        self.x = self.axis[0]
        self.y = self.axis[1]

    def __str__(self):
        return f"Vec2({self.x}, {self.y})"

    def __repr__(self):
        return self.__str__()

    def __eq__(self, other):
        if typeChecker(other, _GeneralVector, error=False):
            return self.x == other.x and self.y == other.y
        else:
            return self.x == other and self.y == other

    def __add__(self, other):
        other = Vec2(other)
        return Vec2(self.x + other.x, self.y + other.y)

    def __neg__(self):
        return Vec2(-self.x, -self.y)

    def __sub__(self, other):
        other = Vec2(other)
        return self + -other

    def __mul__(self, other):
        other = Vec2(other)
        return Vec2(self.x * other.x, self.y * other.y)

    def __truediv__(self, other):
        other = Vec2(other)
        """:param float or Vec2 other:"""
        return Vec2(self.x / other.x, self.y / other.y)

    def __lt__(self, other):
        other = Vec2(other)
        return self.x < other.x and self.y < other.y

    def __gt__(self, other):
        other = Vec2(other)
        return self.x > other.x and self.y > other.y

    def __le__(self, other):
        other = Vec2(other)
        return self.x <= other.x and self.y <= other.y

    def __ge__(self, other):
        other = Vec2(other)
        return self.x >= other.x and self.y >= other.y

    def length(self):
        """
        Get the length of this vector using pythagorean theorem.
        """
        return sqrt(self.x ** 2 + self.y ** 2)

    def normalized(self):
        """
        Get this vector2 normalized by dividing each value by it's length.
        """
        length = self.length()
        if length == 0:
            return Vec2(0)
        return self / length

    def round(self):
        """
        Get this vector2 with each value rounded.
        """
        return Vec2(round(self.x), round(self.y))

    @staticmethod
    def random(value1, value2=None):
        """
        Get a vector2 with random values.

        :param float or Vec2 value1: Minimum value if value2 is specified too. Otherwise it's the maximum value and minimum becomes 0
        :param float or Vec2 value2: Optional maximum value
        """
        if value2 is None:
            value2 = Vec2(value1)
            value1 = Vec2(0)
        else:
            value2 = Vec2(value2)
            value1 = Vec2(value1)
        return Vec2(random.uniform(value1.x, value2.x), random.uniform(value1.y, value2.y))

    def min(self, minimum):
        """
        Get a new vector2 containing the minimum value for each value in the two vectors2.

        :param float or Vec2 minimum: Minimum value, floats are converted to Vec2
        """
        minimum = Vec2(minimum)
        return Vec2(min(self.x, minimum.x), min(self.y, minimum.y))

    def max(self, maximum):
        """
        Get a new vector2 containing the maximum value for each value in the two vectors2.

        :param float or Vec2 maximum: Minimum value, floats are converted to Vec2
        """
        maximum = Vec2(maximum)
        return Vec2(max(self.x, maximum.x), max(self.y, maximum.y))

    def clamp(self, minimum, maximum):
        """
        Get this vector2 clamped between two values.
        :param float or Vec2 minimum: Minimum value, floats are converted to Vec2
        :param float or Vec2 maximum: Maximum value, floats are converted to Vec2
        """
        minimum = Vec2(minimum)
        maximum = Vec2(maximum)
        return Vec2(clamp(self.x, minimum.x, maximum.x), clamp(self.y, minimum.y, maximum.y))

    def inrange(self, minimum, maximum):
        """
        Return whether this vector2 is between two other vector2s.

        :param float or Vec2 minimum: Minimum value, floats are converted to Vec2
        :param float or Vec2 maximum: Maximum value, floats are converted to Vec2
        """
        minimum = Vec2(minimum)
        maximum = Vec2(maximum)
        return inrange(self.x, minimum.x, maximum.x) and inrange(self.y, minimum.y, maximum.y)

    def range(self, size):
        """
        Get a range from two vector2s.
        Self is upper left corner.
        A size of Vec2(1, 1) will return 1 Vec2.

        :param Vec2 size: Size of range.
        :rtype: list[Vec2]
        """
        self.sanitize(ints=True)
        size = Vec2(size).sanitize(ints=True, positiveOrZero=True)

        rangeList = []
        for y in range(size.y):
            for x in range(size.x):
                rangeList.append(self + Vec2(x, y))
        return rangeList

    def distance(self, other):
        """
        Return distance between two Vector2s.

        :param other: Another Vec2
        """
        return (self - other).length()





























from math import sqrt

import random

from generallibrary import clamp, inrange, typeChecker
from generalvector.general import _GeneralVector


class Vec(_GeneralVector):
    """
    Immutable vector.
    """
    def __init__(self, x=None, y=None, z=None):
        _GeneralVector.__init__(self, x, y, z, length=3)

        self.x = self.axis[0]
        self.y = self.axis[1]
        self.z = self.axis[2]

    def __str__(self):
        return f"Vec({self.x}, {self.y}, {self.z})"

    def __repr__(self):
        return self.__str__()

    def __eq__(self, other):
        if typeChecker(other, _GeneralVector, error=False):
            return self.x == other.x and self.y == other.y and self.z == other.z
        else:
            return self.x == other and self.y == other and self.z == other

    def __add__(self, other):
        other = Vec(other)
        return Vec(self.x + other.x, self.y + other.y, self.z + other.z)

    def __neg__(self):
        return Vec(-self.x, -self.y, -self.z)

    def __sub__(self, other):
        return self + -other

    def __mul__(self, other):
        other = Vec(other)
        return Vec(self.x * other.x, self.y * other.y, self.z * other.z)

    def __truediv__(self, other):
        other = Vec(other)
        return Vec(self.x / other.x, self.y / other.y, self.z / other.z)

    def __lt__(self, other):
        other = Vec(other)
        return self.x < other.x and self.y < other.y and self.z < other.z

    def __gt__(self, other):
        other = Vec(other)
        return self.x > other.x and self.y > other.y and self.z > other.z

    def __le__(self, other):
        other = Vec(other)
        return self.x <= other.x and self.y <= other.y and self.z <= other.z

    def __ge__(self, other):
        other = Vec(other)
        return self.x >= other.x and self.y >= other.y and self.z >= other.z

    def length(self):
        """
        Get the length of this vector using pythagorean theorem.
        """
        return sqrt(self.x ** 2 + self.y ** 2 + self.z ** 2)

    def normalized(self):
        """
        Get this vector normalized by dividing each value by it's length.
        """
        length = self.length()
        if length == 0:
            return Vec(0)
        return self / length

    def round(self):
        """
        Get this vector with each value rounded.
        """
        return Vec(round(self.x), round(self.y), round(self.z))

    @staticmethod
    def random(value1, value2=None):
        """
        Get a vector with random values.

        :param float or Vec value1: Minimum value if value2 is specified too. Otherwise it's the maximum value and minimum becomes 0
        :param float or Vec value2: Optional maximum value
        """
        if value2 is None:
            value2 = Vec(value1)
            value1 = Vec(0)
        else:
            value2 = Vec(value2)
            value1 = Vec(value1)
        return Vec(random.uniform(value1.x, value2.x), random.uniform(value1.y, value2.y), random.uniform(value1.z, value2.z))

    def min(self, minimum):
        """
        Get a new vector containing the minimum value for each value in the two vectors.

        :param float or Vec minimum: Minimum value, floats are converted to Vec
        """
        minimum = Vec(minimum)
        return Vec(min(self.x, minimum.x), min(self.y, minimum.y), min(self.z, minimum.z))

    def max(self, maximum):
        """
        Get a new vector containing the maximum value for each value in the two vectors.

        :param float or Vec maximum: Minimum value, floats are converted to Vec
        """
        maximum = Vec(maximum)
        return Vec(max(self.x, maximum.x), max(self.y, maximum.y), max(self.z, maximum.z))

    def clamp(self, minimum, maximum):
        """
        Get this vector clamped between two values as a new vector.

        :param float or Vec minimum: Minimum value, floats are converted to Vec
        :param float or Vec maximum: Maximum value, floats are converted to Vec
        """
        minimum = Vec(minimum)
        maximum = Vec(maximum)
        return Vec(clamp(self.x, minimum.x, maximum.x), clamp(self.y, minimum.y, maximum.y), clamp(self.z, minimum.z, maximum.z))

    def inrange(self, minimum, maximum):
        """
        Return whether this vector is between two other vectors.

        :param float or Vec minimum: Minimum value, floats are converted to Vec
        :param float or Vec maximum: Maximum value, floats are converted to Vec
        """
        return inrange(self.x, minimum.x, maximum.x) and inrange(self.y, minimum.y, maximum.y) and inrange(self.z, minimum.z, maximum.z)

    def hex(self):
        """
        Get a hex based on each value. Rounded and clamped between 0 and 255.
        """
        rounded = self.round().clamp(0, 255)
        return f"#{'%02x' % rounded.x}{'%02x' % rounded.y}{'%02x' % rounded.z}"

    def range(self, size):
        """
        Get a range from two vectors.
        Self is upper left corner.
        A size of Vec(1, 1) will return 1 Vec.

        :param Vec size: Size of range.
        :rtype: list[Vec]
        """
        self.sanitize(ints=True)
        size = Vec(size).sanitize(ints=True, positiveOrZero=True)

        rangeList = []
        for z in range(size.z):
            for y in range(size.y):
                for x in range(size.x):
                    rangeList.append(self + Vec(x, y, z))
        return rangeList

    def distance(self, other):
        """
        Return distance between two Vectors.

        :param other: Another Vec
        """
        return (self - other).length()













